from django.test import TestCase
from django.core.management import call_command

from . import utils
from .models import TestModel
from ..trackers import track_model


class ReTransformImagesTest(utils.RemoveStorage, TestCase):
    def setUp(self):
        self.default_dimensions = (200, 200)
        self.image_name = 'image.jpg'
        self.disconnect = track_model(TestModel)
        self.model = TestModel.objects.create()
        self.model.image.save(
            self.image_name,
            utils.django_image(
                *self.default_dimensions,
                name=self.image_name
            )
        )

        self.model.thumbnail.save(
            self.image_name,
            utils.django_image(
                *self.default_dimensions,
                name=self.image_name
            ),
            save=False
        )

    def tearDown(self):
        self.disconnect()

    def test_thumb_wrong(self):
        self.assertEqual(
            self.model.thumbnail.height,
            self.default_dimensions[0],
        )

    def test_retransform_specific_model(self):
        call_command('retransform', 'test.TestModel')
        self.model_non_cached = TestModel.objects.get(pk=self.model.pk)
        self.assertEqual(
            self.model_non_cached.thumbnail.width,
            10,
        )

    def test_retransform_specific_field(self):
        call_command('retransform', 'test.TestModel.image')
        self.model_non_cached = TestModel.objects.get(pk=self.model.pk)
        self.assertEqual(
            self.model_non_cached.thumbnail.width,
            10,
        )



from StringIO import StringIO

from PIL import Image

from django.core.files.uploadedfile import InMemoryUploadedFile
from django.core.files import File


def scale(height=None, width=None):
    def scale_image_function(django_file):
        pil_image = pil_image_from_django_file(django_file)
        max_height = height or pil_image.size[0]
        max_width = width or pil_image.size[1]
        dimensions = (max_height, max_width)
        transformed_pil_image = pil_image.copy()
        transformed_pil_image.thumbnail(dimensions, Image.ANTIALIAS)
        tranformed_django_file = django_file_from_pil_image(
            transformed_pil_image,
            django_file.name
        )
        return tranformed_django_file
    return scale_image_function


def pil_image_from_django_file(django_file):
    if not isinstance(django_file, File):
        raise TypeError(
            'image is a {0}, not a django File'.format(type(django_file))
        )
    if not django_file.file:
        raise ValueError(
            'the django file has no file saved to it.'
        )
    django_file.open()
    return Image.open(django_file.file)


def django_file_from_pil_image(transformed_pil_image, file_name):
    if not isinstance(transformed_pil_image, Image.Image):
        raise TypeError(
            'image is a {0}, not a PIL Image'.format(type(transformed_pil_image))
        )
    temp_io = StringIO()
    transformed_pil_image.save(temp_io, format='JPEG')
    temp_io.seek(0)
    django_file = InMemoryUploadedFile(
        file=temp_io,
        field_name=None,
        name=file_name,
        content_type='image/jpeg',
        size=temp_io.len,
        charset=None,
    )
    return django_file



import StringIO
from subprocess import call

from PIL import Image

from django.core.files.uploadedfile import InMemoryUploadedFile


def image(width=10, height=10):
    thumb = pil_image(width, height)
    # Create a file-like object to write thumb data (thumb data previously created
    # using PIL, and stored in variable 'thumb')
    thumb_io = StringIO.StringIO()
    thumb.save(thumb_io, format='JPEG')
    thumb_io.seek(0)
    return thumb_io


def pil_image(width=10, height=10):
    return Image.new('RGB', (width, height,), 'blue')


def django_image(height=10, width=10, name='name'):
    image_file = image(width, height)
    # Create a new Django file-like object to be used in models as ImageField using
    # InMemoryUploadedFile.  If you look at the source in Django, a
    # SimpleUploadedFile is essentially instantiated similarly to what is shown here
    return InMemoryUploadedFile(image_file, None, name + '.jpg', 'image/jpeg',
                                image_file.len, None)


class RemoveStorage(object):
    def tearDown(self):
        call('rm -rf testing', shell=True)



from django.db import models

from ..transforms import scale


class TestModel(models.Model):
    image = models.ImageField(
        upload_to='testing/'
    )
    thumbnail = models.ImageField(
        blank=True,
        null=True,
        editable=False,
        upload_to='testing/thumbnails/'
    )

    transformed_fields = {
        'image': {
            'thumbnail': scale(width=10),
        }
    }



def perform_transformation(instances, field_names=None):
    '''
    Transforms the images, from the field_name to any fields that are transformed
    from that field. Will transform the images in each of the instances. By
    default it will not save the model after transforming. Also if the
    `field_names` is false, it will resave all the fields on the model.
    '''
    for instance in instances:
        updated_fields = []
        transformed_fields_dict = instance.transformed_fields
        for original_field_name, destination_dict in transformed_fields_dict.items():
            if field_names and not original_field_name in field_names:
                break
            original_field = getattr(instance, original_field_name)
            if not original_field:
                break
            original_name = original_field.name

            for destination_field_name, transformation in destination_dict.items():
                destination_field = getattr(instance, destination_field_name)
                new_image = transformation(original_field)
                destination_field.save(
                    original_name,
                    new_image,
                    save=False
                )
                updated_fields.append(destination_field_name)
        instance.save(update_fields=updated_fields)



from django.db.models.signals import pre_save
from django.dispatch import receiver

from . import utils


def track_model(model):
    @receiver(pre_save, sender=model, weak=False, dispatch_uid='simpleimages')
    def transform_signal(sender, **kwargs):
        utils.perform_transformation(
            [kwargs['instance']],
            kwargs['update_fields']
        )

    def disconnect():
        pre_save.disconnect(sender=model, dispatch_uid='simpleimages')
    return disconnect



from django.test import TestCase

from ..utils import perform_transformation
from . import utils
from .models import TestModel


class PerformTransformationTest(utils.RemoveStorage, TestCase):
    def setUp(self):
        self.default_dimensions = (200, 200)
        self.image_name = 'image.jpg'

        self.model = TestModel.objects.create()
        self.model2 = TestModel.objects.create()
        self.model.image.save(
            self.image_name,
            utils.django_image(
                *self.default_dimensions,
                name=self.image_name
            )
        )
        self.model2.image.save(
            '2' + self.image_name,
            utils.django_image(
                *self.default_dimensions,
                name='2' + self.image_name
            )
        )

    def test_initial_blank(self):
        self.assertFalse(self.model.thumbnail)
        self.assertFalse(self.model2.thumbnail)

    def test_transform_creates_another_image(self):
        perform_transformation([self.model])

        self.non_cached_model = TestModel.objects.get(pk=self.model.pk)
        self.non_cached_model2 = TestModel.objects.get(pk=self.model2.pk)

        self.assertTrue(self.model.thumbnail)
        self.assertFalse(self.model2.thumbnail)
        self.assertTrue(self.non_cached_model.thumbnail)
        self.assertFalse(self.non_cached_model2.thumbnail)

    def test_transformed_image_is_different(self):
        perform_transformation([self.model])

        self.non_cached_model = TestModel.objects.get(pk=self.model.pk)

        self.assertEqual(self.model.thumbnail.width, 10)
        self.assertEqual(self.non_cached_model.thumbnail.width, 10)

    def test_transform_two_instances(self):
        perform_transformation([self.model, self.model2])

        self.non_cached_model = TestModel.objects.get(pk=self.model.pk)
        self.non_cached_model2 = TestModel.objects.get(pk=self.model2.pk)

        self.assertTrue(self.model.thumbnail)
        self.assertTrue(self.model2.thumbnail)
        self.assertTrue(self.non_cached_model.thumbnail)
        self.assertTrue(self.non_cached_model2.thumbnail)

    def test_transform_queryset(self):
        perform_transformation(TestModel.objects.all())

        self.non_cached_model = TestModel.objects.get(pk=self.model.pk)
        self.non_cached_model2 = TestModel.objects.get(pk=self.model2.pk)

        self.assertTrue(self.non_cached_model.thumbnail)
        self.assertTrue(self.non_cached_model2.thumbnail)

    def test_transform_wrong_field(self):
        perform_transformation([self.model], field_names=['sdf'])

        self.non_cached_model = TestModel.objects.get(pk=self.model.pk)

        self.assertFalse(self.model.thumbnail)
        self.assertFalse(self.non_cached_model.thumbnail)

    def test_transform_right_field(self):
        perform_transformation([self.model], field_names=['image'])

        self.non_cached_model = TestModel.objects.get(pk=self.model.pk)

        self.assertTrue(self.model.thumbnail)
        self.assertTrue(self.non_cached_model.thumbnail)



from PIL import Image

from django.test import TestCase
from django.core.files.images import get_image_dimensions
from django.core.files import File

from simpleimages.transforms import (scale, pil_image_from_django_file,
                                     django_file_from_pil_image)
from . import utils


class ConversionTest(TestCase):

    def test_pil_from_djange(self):
        django_image = utils.django_image(width=100, height=100)
        pil_image = pil_image_from_django_file(django_image)
        self.assertIsInstance(pil_image, Image.Image)
        self.assertItemsEqual(pil_image.size, [100, 100])

    def test_django_from_pil(self):
        pil_image = utils.pil_image(width=100, height=100)
        django_file = django_file_from_pil_image(pil_image, 'image.jpg')
        self.assertIsInstance(django_file, File)


class ScaleTest(utils.RemoveStorage, TestCase):
    def test_width(self):
        '''
        Make sure that if width shrinks, then height shrinks
        proportionally
        '''
        old_image = utils.django_image(width=100, height=100)
        transform = scale(width=10)
        new_image = transform(old_image)
        new_height, new_width = get_image_dimensions(new_image)
        self.assertEqual(new_height, 10)
        self.assertEqual(new_width, 10)

    def test_over_large(self):
        '''
        if specified dimension is larger than image, it shouldn't enlarge
        the image
        '''
        old_image = utils.django_image(width=100, height=100)
        transform = scale(width=200)
        new_image = transform(old_image)
        new_height, new_width = get_image_dimensions(new_image)
        self.assertEqual(new_height, 100)
        self.assertEqual(new_width, 100)



from setuptools import setup

setup(
    name='django-simpleimages',
    version='0.1.3',
    author='Saul Shanabrook',
    author_email='s.shanabrook@gmail.com',
    packages=['simpleimages', 'simpleimages.test'],
    url='https://www.github.com/saulshanabrook/django-simpleimages',
    license='LICENSE.txt',
    description='Opinionated Django image transforms on models',
    long_description=open('README.rst').read(),
    install_requires=[
        "Django >= 1.5",
    ],
)
